/**
 * @file 	RLEregion.h
 * @brief	Header file for RLEregion.c. 
 *		Structure that is used to represent a region, used for 
 *		rle segmentation.
 * @date	Thu Mar 29 16:41:15  2001 - Original.
 * @author	Binh Bui (Binh.Bui@sophia.inria.fr).
 */


#ifndef _RLEREGION_H_
#define _RLEREGION_H_

// #include "sup/CSmartPointer.h"
#include <tr1/memory>

#include <vector>

#include <RLEimage.h>

namespace vsip {

struct blob;

/** a blob_t is pointer to a blob structure. */
typedef std::tr1::shared_ptr < struct RLEblob > blob_t;

/** region_t is a pointer to a region structure. */
typedef std::tr1::shared_ptr < struct region > region_t;

/** structure region, a region is a set of connected segments */
struct region {
    /** X coordinate of the upper left point belong to region. */
    int ulx;         
    
    /** 
     * Y coordinate of the upper left point belong to region. It is also the
     * minimal y of region's bounding box.
     */
    int uly;		
    
    /** Identifier */
    int id;	
    
    /** Number of pixels */
    int size;
    
    /** Number of holes this region contains. Actually, not used. */
    int nbholes;
    
    /** @{ */
    /** Bouding box coordinate. */
    int xmin, xmax, ymax;	
    /** @} */
    
    /** 
     * Sum of x of all pixels in this region, used to compute the center of 
     * gravity of this region. 
     */
    double sumx;
    
    /** 
     * Sum of y of all pixels in this region, used to compute the center of 
     * gravity of this region. 
     */
    double sumy;
    
    /** 
     * Blob to which the region belong, usefull for background updating. If this
     * region not belong to a blob then this attribut = NULL.
     */
    blob_t blob;
    
    /** @{ */
    /** Pointer to neighbor region. */
    region_t next;
    region_t up;
    /** @} */
    
    /** Pointer to the hole this region contain (if exist). Actually, not used. */
    region_t inside;

  //Added by Marcos Zuniga for blob vertical split
  /** Array of segment lines belonging to a region */
  std::vector < RLEregionline_t > region_segment_lines;

};




/** segment structure. */
struct segment {
	int	xright;			/**< x left */
	int xleft;			/**< x right */
	region_t region;		/**< region contains this segment */
};

/** a segment_t is a pointer to a segment structure */
typedef std::tr1::shared_ptr < struct segment > segment_t;

struct segment_list
{
  struct segment segment;
  struct segment_list *next;
};

typedef struct segment_list *segment_list_t;

#define SEGMENT_LIST_SEGMENT(sl)        (&((sl)->segment))
#define SEGMENT_LIST_XLEFT(sl)          (((sl)->segment).xleft)
#define SEGMENT_LIST_XRIGHT(sl)         (((sl)->segment).xright)
#define SEGMENT_LIST_REGION(sl)         (((sl)->segment).region)
#define SEGMENT_LIST_NEXT(sl)           ((sl)->next)

/**
 * Allocate memory for a segment.
 * @return Address of the new segment.
 */
segment_t segment_new();


/**
 * Allocate memory for a segment pool. A segment pool is a array of successive
 * segments.
 * @param size number of segment in pool.
 * @return Address of the first segment in pool.
 */
segment_t segment_new_pool(int size);
  

/**
 * free a segment (or a segment pool).
 */
void segment_free(segment_t seg);

/** Modifications added by Christophe Tornieri 
 *  The segment list is used for extra segments. These extra segments
 *  are used to prevent updating part of reference image belonging to blob.
 */ 

/**
 * Create a new empty segment list
 */
segment_list_t new_segment_list();

/**
 * Free the segment list
 */
void free_segment_list(segment_list_t segment_list);

/**
 * Insert the element in the list at the right place
 * Before the insertion list is ordered on xleft component.
 * After the insertion list is ordered on xleft component.
 */
segment_list_t insert_in_seg_list(segment_list_t segment_list, int xleft, int xright, region_t region);

/**
 * Allocate memory for a region.
 * @return Address of the new region.
 */
region_t region_new();
    

/**
 * Allocate memory for a region pool. A region pool is a array of successive 
 * regions.
 * @param size number of regions in pool.
 * @return Address of the first region in pool.
 */
region_t region_new_pool(int size);


/**
 * free a region (or a region pool).
 * @param region region (region pool) to free.
 */
void region_free(region_t region);


/**
 * initialize a region from a segment.
 * @param region region.
 * @xleft left x of segment.
 * @xright right x of segment.
 * @y coordinate y of segment.
 */
void 
region_initialize(region_t	region, 
		  int		xleft, 
		  int		xright,
		  int 		y);  


/**
 * Absorb a segment into a region.
 * @param region target region.
 * @param xleft x left of segment.
 * @param xright x right of segment.
 * @param y coordinate y of segment.
 * @note The attributs of region will be updated
 */
void 
region_absorb_segment(region_t 	region, 
		      int 	xleft, 
		      int	xright,
		      int 	y);    


/**
 * Absorb a region into a region.
 * @param region target region.
 * @param absorbed region will be absorbed.
 * @note The attributs of region will be updated.
 */
void 
region_absorb_region(region_t 	region, 
		     region_t 	absorbed);


/**
 * add a hole region to a region.
 * @param region region contains the hole.
 * @param hole hole region.
 */      
void 
region_add_hole(region_t region, 
		region_t hole);

} // end namespace vsip		
			 
#endif /* _RLEREGION_H_ */

